@@ -43,10 +43,16 @@
 #endif
 
 #include <su/cs.h>
+#include <su/mem.h>
 #include <su/utf.h>
 
+#include "mx/child.h"
+#include "mx/dig-msg.h"
+#include "mx/file-streams.h"
 #include "mx/filter-quote.h"
 #include "mx/names.h"
+#include "mx/sigs.h"
+#include "mx/tty.h"
 #include "mx/ui-str.h"
 
 /* TODO fake */
@@ -67,9 +73,9 @@ struct a_coll_fmt_ctx{ /* xxx This is temporary until v15 has objects */
 struct a_coll_ocs_arg{
    n_sighdl_t coa_opipe;
    n_sighdl_t coa_oint;
-   FILE *coa_stdin;  /* The descriptor (pipe(2)+Fdopen()) we read from */
-   FILE *coa_stdout; /* The Popen()ed pipe through which we write to the hook */
-   int coa_pipe[2];  /* ..backing .coa_stdin */
+   FILE *coa_stdin; /* The descriptor (pipe(2)+fs_fd_open()) we read from */
+   FILE *coa_stdout; /* The pipe_open() through which we write to the hook */
+   sz coa_pipe[2]; /* ..backing .coa_stdin */
    s8 *coa_senderr; /* Set to 1 on failure */
    char coa_cmd[VFIELD_SIZE(0)];
 };
@@ -208,8 +214,7 @@ a_coll_include_file(char const *name, boole indent, boole writestat){
 
    rv = su_ERR_NONE;
    lc = cc = 0;
-   linebuf = NULL; /* TODO line pool */
-   linesize = 0;
+   mx_fs_linepool_aquire(&linebuf, &linesize);
    heredb = NULL;
    heredl = 0;
 
@@ -256,7 +261,7 @@ jdelim_empty:
          }
       }
       name = n_hy;
-   }else if((fbuf = Fopen(name, "r")) == NULL){
+   }else if((fbuf = mx_fs_open(name, "r")) == NIL){
       n_perr(name, rv = su_err_no());
       goto jleave;
    }
@@ -296,11 +301,10 @@ jdelim_empty:
    if(heredb != NULL)
       rv = su_ERR_NOTOBACCO;
 jleave:
-   if(linebuf != NULL)
-      n_free(linebuf);
-   if(fbuf != NULL){
+   mx_fs_linepool_release(linebuf, linesize);
+   if(fbuf != NIL){
       if(fbuf != n_stdin)
-         Fclose(fbuf);
+         mx_fs_close(fbuf);
       else if(heredl > 0)
          clearerr(n_stdin);
    }
@@ -322,7 +326,7 @@ a_coll_insert_cmd(FILE *fp, char const *cmd){
    rv = su_ERR_NONE;
    lc = cc = 0;
 
-   if((ibuf = Popen(cmd, "r", ok_vlook(SHELL), NULL, 0)) != NULL){
+   if((ibuf = mx_fs_pipe_open(cmd, "r", ok_vlook(SHELL), NIL, -1)) != NIL){
       int c;
 
       while((c = getc(ibuf)) != EOF){ /* XXX bytewise, yuck! */
@@ -338,7 +342,7 @@ a_coll_insert_cmd(FILE *fp, char const *cmd){
          if(rv == su_ERR_NONE)
             rv = su_ERR_IO;
       }
-      if(!Pclose(ibuf, TRU1)){
+      if(!mx_fs_pipe_close(ibuf, TRU1)){
          if(rv == su_ERR_NONE)
             rv = su_ERR_IO;
       }
@@ -360,14 +364,15 @@ print_collf(FILE *cf, struct header *hp)
    NYD_IN;
 
    fflush_rewind(cf);
-   cnt = (uz)fsize(cf);
+   cnt = S(uz,fsize(cf));
 
-   if((obuf = Ftmp(NULL, "collfp", OF_RDWR | OF_UNLINK | OF_REGISTER)) == NULL){
+   if((obuf = mx_fs_tmp_open("collfp", (mx_FS_O_RDWR | mx_FS_O_UNLINK |
+            mx_FS_O_REGISTER), NIL)) == NIL){
       n_perr(_("Can't create temporary file for `~p' command"), 0);
       goto jleave;
    }
 
-   hold_all_sigs();
+   mx_sigs_all_holdx();
 
    fprintf(obuf, _("-------\nMessage contains:\n")); /* XXX112 */
    n_puthead(TRU1, hp, obuf,
@@ -386,11 +391,11 @@ print_collf(FILE *cf, struct header *hp)
       n_attachment_list_print(hp->h_attach, obuf);
    }
 
-   rele_all_sigs();
+   mx_sigs_all_rele();
 
    page_or_print(obuf, 0);
 
-   Fclose(obuf);
+   mx_fs_close(obuf);
 jleave:
    NYD_OU;
 }
@@ -411,7 +416,7 @@ a_coll_write(char const *name, FILE *fp, int f)
       fflush(n_stdout);
    }
 
-   if ((of = Fopen(name, "a")) == NULL) {
+   if((of = mx_fs_open(name, "a")) == NIL){
       n_perr(name, rv = su_err_no());
       goto jerr;
    }
@@ -429,8 +434,8 @@ a_coll_write(char const *name, FILE *fp, int f)
    fprintf(n_stdout, "%" PRId64 "/%" PRId64 "\n", lc, cc);
 
 jleave:
-   if(of != NULL)
-      Fclose(of);
+   if(of != NIL)
+      mx_fs_close(of);
    fflush(n_stdout);
    NYD_OU;
    return rv;
@@ -482,7 +487,7 @@ a_coll_quote_message(FILE *fp, struct message *mp, boole isfwd){
                "*fwdheading*"));
          if((cp = ok_vlook(forward_inject_head)) == NULL &&
                (cp = cp_v15compat) == NULL)
-            cp = n_FORWARD_INJECT_HEAD;
+            cp = n_FORWARD_INJECT_HEAD; /* v15compat: make auto-defval */
          quoteitp = n_IGNORE_FWD;
       }else{
          if(!su_cs_cmp(cp, "noheading")){
@@ -495,7 +500,7 @@ a_coll_quote_message(FILE *fp, struct message *mp, boole isfwd){
             action = SEND_QUOTE_ALL;
             cp = NULL;
          }else if((cp = ok_vlook(quote_inject_head)) == NULL)
-            cp = n_QUOTE_INJECT_HEAD;
+            cp = n_QUOTE_INJECT_HEAD; /* v15compat: make auto-defval */
       }
       /* We we pass through our formatter? */
       if((cfc.cfc_fmt = cp) != NULL){
@@ -615,7 +620,8 @@ a_coll_makeheader(FILE *fp, struct header *hp, s8 *checkaddr_err,
 
    rv = FAL0;
 
-   if ((nf = Ftmp(NULL, "colhead", OF_RDWR | OF_UNLINK | OF_REGISTER)) ==NULL) {
+   if((nf = mx_fs_tmp_open("colhead", (mx_FS_O_RDWR | mx_FS_O_UNLINK |
+            mx_FS_O_REGISTER), NIL)) == NIL){
       n_perr(_("temporary mail edit file"), 0);
       goto jleave;
    }
@@ -645,18 +651,18 @@ a_coll_makeheader(FILE *fp, struct header *hp, s8 *checkaddr_err,
    while ((c = getc(fp)) != EOF) /* XXX bytewise, yuck! */
       putc(c, nf);
 
-   if (fp != _coll_fp)
-      Fclose(_coll_fp);
-   Fclose(fp);
+   if(fp != _coll_fp)
+      mx_fs_close(_coll_fp);
+   mx_fs_close(fp);
    _coll_fp = nf;
-   nf = NULL;
+   nf = NIL;
 
    if (check_from_and_sender(hp->h_from, hp->h_sender) == NULL)
       goto jleave;
    rv = TRU1;
 jleave:
-   if(nf != NULL)
-      Fclose(nf);
+   if(nf != NIL)
+      mx_fs_close(nf);
    NYD_OU;
    return rv;
 }
@@ -673,7 +679,6 @@ a_coll_edit(int c, struct header *hp, char const *pipecmd) /* TODO errret */
    NYD_IN;
 
    rv = su_ERR_NONE;
-   UNINIT(sigint, SIG_ERR);
    saved_filrec = ok_blook(add_file_recipients);
 
    n_SIGMAN_ENTER_SWITCH(&sm, n_SIGMAN_ALL){
@@ -681,6 +686,7 @@ a_coll_edit(int c, struct header *hp, char const *pipecmd) /* TODO errret */
       sigint = safe_signal(SIGINT, SIG_IGN);
       break;
    default:
+      sigint = SIG_ERR;
       rv = su_ERR_INTR;
       goto jleave;
    }
@@ -718,7 +724,7 @@ a_coll_edit(int c, struct header *hp, char const *pipecmd) /* TODO errret */
          }
       }else{
          fseek(nf, 0L, SEEK_END);
-         Fclose(_coll_fp);
+         mx_fs_close(_coll_fp);
          _coll_fp = nf;
       }
    }else
@@ -728,7 +734,8 @@ a_coll_edit(int c, struct header *hp, char const *pipecmd) /* TODO errret */
 jleave:
    if(!saved_filrec)
       ok_bclear(add_file_recipients);
-   safe_signal(SIGINT, sigint);
+   if(sigint != SIG_ERR)
+      safe_signal(SIGINT, sigint);
    NYD_OU;
    n_sigman_leave(&sm, n_SIGMAN_VIPSIGS_NTTYOUT);
    return rv;
@@ -737,7 +744,6 @@ jleave:
 static s32
 a_coll_pipe(char const *cmd)
 {
-   int ws;
    FILE *nf;
    n_sighdl_t sigint;
    s32 rv;
@@ -746,7 +752,8 @@ a_coll_pipe(char const *cmd)
    rv = su_ERR_NONE;
    sigint = safe_signal(SIGINT, SIG_IGN);
 
-   if ((nf = Ftmp(NULL, "colpipe", OF_RDWR | OF_UNLINK | OF_REGISTER)) ==NULL) {
+   if((nf = mx_fs_tmp_open("colpipe", (mx_FS_O_RDWR | mx_FS_O_UNLINK |
+            mx_FS_O_REGISTER), NIL)) == NIL){
 jperr:
       n_perr(_("temporary mail edit file"), rv = su_err_no());
       goto jout;
@@ -756,23 +763,34 @@ jperr:
    if(fflush(_coll_fp) == EOF)
       goto jperr;
    rewind(_coll_fp);
-   if (n_child_run(ok_vlook(SHELL), 0, fileno(_coll_fp), fileno(nf), "-c",
-         cmd, NULL, NULL, &ws) < 0 || WEXITSTATUS(ws) != 0) {
-      Fclose(nf);
-      rv = su_ERR_CHILD;
-      goto jout;
+
+   /* C99 */{
+      struct mx_child_ctx cc;
+
+      mx_child_ctx_setup(&cc);
+      cc.cc_flags = mx_CHILD_RUN_WAIT_LIFE;
+      cc.cc_fds[mx_CHILD_FD_IN] = fileno(_coll_fp);
+      cc.cc_fds[mx_CHILD_FD_OUT] = fileno(nf);
+      cc.cc_cmd = ok_vlook(SHELL);
+      cc.cc_args[0] = "-c";
+      cc.cc_args[1] = cmd;
+      if(!mx_child_run(&cc) || cc.cc_exit_status != 0){
+         mx_fs_close(nf);
+         rv = su_ERR_CHILD;
+         goto jout;
+      }
    }
 
    if (fsize(nf) == 0) {
       n_err(_("No bytes from %s !?\n"), n_shexp_quote_cp(cmd, FAL0));
-      Fclose(nf);
+      mx_fs_close(nf);
       rv = su_ERR_NODATA;
       goto jout;
    }
 
    /* Take new files */
    fseek(nf, 0L, SEEK_END);
-   Fclose(_coll_fp);
+   mx_fs_close(_coll_fp);
    _coll_fp = nf;
 jout:
    safe_signal(SIGINT, sigint);
@@ -918,10 +936,10 @@ a_coll_ocs__mac(void){
    /* Executes in a fork(2)ed child  TODO if remains, global MASKs for those! */
    setvbuf(n_stdin, NULL, _IOLBF, 0);
    setvbuf(n_stdout, NULL, _IOLBF, 0);
-   n_psonce &= ~(n_PSO_INTERACTIVE | n_PSO_TTYIN | n_PSO_TTYOUT);
+   n_psonce &= ~(n_PSO_INTERACTIVE | n_PSO_TTYANY);
    n_pstate |= n_PS_COMPOSE_FORKHOOK;
    n_readctl_read_overlay = NULL; /* TODO need OnForkEvent! See c_readctl() */
-   n_digmsg_read_overlay = NULL; /* TODO need OnForkEvent! See c_digmsg() */
+   mx_dig_msg_read_overlay = NIL; /* TODO need OnForkEvent! See c_digmsg() */
    if(n_poption & n_PO_D_VV){
       char buf[128];
 
@@ -946,25 +964,25 @@ a_coll_ocs__finalize(void *vp){
    coap = *(coapp = vp);
    *coapp = (struct a_coll_ocs_arg*)-1;
 
-   if(coap->coa_stdin != NULL)
-      Fclose(coap->coa_stdin);
+   if(coap->coa_stdin != NIL)
+      mx_fs_close(coap->coa_stdin);
    else if(coap->coa_pipe[0] != -1)
-      close(coap->coa_pipe[0]);
+      close(S(int,coap->coa_pipe[0]));
 
-   if(coap->coa_stdout != NULL && !Pclose(coap->coa_stdout, TRU1))
-      *coap->coa_senderr = 111;
+   if(coap->coa_stdout != NIL && !mx_fs_pipe_close(coap->coa_stdout, TRU1))
+      *coap->coa_senderr = 1;
    if(coap->coa_pipe[1] != -1)
-      close(coap->coa_pipe[1]);
+      close(S(int,coap->coa_pipe[1]));
 
    opipe = coap->coa_opipe;
    oint = coap->coa_oint;
 
    n_lofi_free(coap);
 
-   hold_all_sigs();
+   mx_sigs_all_holdx();
    safe_signal(SIGPIPE, opipe);
    safe_signal(SIGINT, oint);
-   rele_all_sigs();
+   mx_sigs_all_rele();
    NYD2_OU;
 }
 
@@ -1028,39 +1046,48 @@ FL FILE *
 n_collect(enum n_mailsend_flags msf, struct header *hp, struct message *mp,
    char const *quotefile, s8 *checkaddr_err)
 {
-   struct n_dig_msg_ctx dmc;
-   struct n_string s_b, * volatile s;
-   struct a_coll_ocs_arg *coap;
-   int c;
-   int volatile t, eofcnt, getfields;
-   char volatile escape;
    enum{
       a_NONE,
       a_ERREXIT = 1u<<0,
       a_IGNERR = 1u<<1,
-      a_COAP_NOSIGTERM = 1u<<8
 #define a_HARDERR() ((flags & (a_ERREXIT | a_IGNERR)) == a_ERREXIT)
-   } volatile flags;
+
+      a_COAP_NOSIGTERM = 1u<<8,
+
+      a_NEED_INJECT_RESTART = 1u<<16,
+      a_CAN_DELAY_INJECT = 1u<<17,
+      a_EVER_LEFT_INPUT_LOOPS = 1u<<18,
+      a_ROUND_MASK = a_NEED_INJECT_RESTART | a_CAN_DELAY_INJECT |
+            a_EVER_LEFT_INPUT_LOOPS
+   };
+
+   struct mx_dig_msg_ctx dmc;
+   struct n_string s_b, * volatile s;
+   struct a_coll_ocs_arg *coap;
+   int c;
+   int volatile gfield, eofcnt, getfields;
+   char volatile escape;
    char *linebuf;
    char const *cp, *cp_base, * volatile coapm, * volatile ifs_saved;
-   uz i, linesize; /* TODO line pool */
+   uz i, linesize;
    long cnt;
    sigset_t oset, nset;
+   u32 volatile flags;
    FILE * volatile sigfp;
    NYD_IN;
 
-   n_DIG_MSG_COMPOSE_CREATE(&dmc, hp);
+   mx_DIG_MSG_COMPOSE_CREATE(&dmc, hp);
    _coll_fp = NULL;
 
    sigfp = NULL;
-   linesize = 0;
-   linebuf = NULL;
-   flags = a_NONE;
+   flags = a_CAN_DELAY_INJECT;
    eofcnt = 0;
    ifs_saved = coapm = NULL;
    coap = NULL;
    s = NULL;
 
+   mx_fs_linepool_aquire(&linebuf, &linesize);
+
    /* Start catching signals from here, but we still die on interrupts
     * until we're in the main loop */
    sigfillset(&nset);
@@ -1076,8 +1103,8 @@ n_collect(enum n_mailsend_flags msf, struct header *hp, struct message *mp,
    n_pstate |= n_PS_COMPOSE_MODE;
    sigprocmask(SIG_SETMASK, &oset, (sigset_t*)NULL);
 
-   if ((_coll_fp = Ftmp(NULL, "collect", OF_RDWR | OF_UNLINK | OF_REGISTER)) ==
-         NULL) {
+   if((_coll_fp = mx_fs_tmp_open("collect", (mx_FS_O_RDWR | mx_FS_O_UNLINK |
+            mx_FS_O_REGISTER), NIL)) == NIL){
       n_perr(_("collect: temporary mail file"), 0);
       goto jerr;
    }
@@ -1086,33 +1113,33 @@ n_collect(enum n_mailsend_flags msf, struct header *hp, struct message *mp,
     * after the headers (since some people mind) */
    getfields = 0;
    if(!(n_poption & n_PO_t_FLAG)){
-      t = GTO | GSUBJECT | GCC | GNL;
+      gfield = GTO | GSUBJECT | GCC | GBCC | GNL;
       if(ok_blook(fullnames))
-         t |= GCOMMA;
+         gfield |= GCOMMA;
 
       if((n_psonce & n_PSO_INTERACTIVE) && !(n_pstate & n_PS_ROBOT)){
-         if(hp->h_subject == NULL && ok_blook(asksub)/* *ask* auto warped! */)
-            t &= ~GNL, getfields |= GSUBJECT;
+         if(hp->h_subject == NIL && ok_blook(asksub)/* *ask* auto warped! */)
+            gfield &= ~(GSUBJECT | GNL), getfields |= GSUBJECT;
 
          if(hp->h_to == NULL)
-            t &= ~GNL, getfields |= GTO;
+            gfield &= ~(GTO | GNL), getfields |= GTO;
 
          if(!ok_blook(bsdcompat) && !ok_blook(askatend)){
             if(ok_blook(askbcc))
-               t &= ~GNL, getfields |= GBCC;
+               gfield &= ~(GBCC | GNL), getfields |= GBCC;
             if(ok_blook(askcc))
-               t &= ~GNL, getfields |= GCC;
+               gfield &= ~(GCC | GNL), getfields |= GCC;
          }
       }
    }else{
-      UNINIT(t, 0);
+      UNINIT(gfield, 0);
    }
 
    _coll_hadintr = 0;
 
    if (!sigsetjmp(_coll_jmp, 1)) {
       /* Ask for some headers first, as necessary */
-      if (getfields)
+      if(getfields)
          grab_headers(n_GO_INPUT_CTX_COMPOSE, hp, getfields, 1);
 
       /* Execute compose-enter; delayed for -t mode */
@@ -1148,26 +1175,36 @@ n_collect(enum n_mailsend_flags msf, struct header *hp, struct message *mp,
 
          if(!(n_poption & n_PO_Mm_FLAG) && !(n_pstate & n_PS_ROBOT)){
             /* Print what we have sofar also on the terminal (if useful) */
-            if((cp = ok_vlook(editalong)) == NULL){
-               if(msf & n_MAILSEND_HEADERS_PRINT)
-                  n_puthead(TRU1, hp, n_stdout, t, SEND_TODISP, CONV_NONE,
-                     NULL, NULL);
-
-               rewind(_coll_fp);
-               while ((c = getc(_coll_fp)) != EOF) /* XXX bytewise, yuck! */
-                  putc(c, n_stdout);
-               if (fseek(_coll_fp, 0, SEEK_END))
-                  goto jerr;
+            if(n_go_input_have_injections()){
+               flags |= a_NEED_INJECT_RESTART;
+               flags &= ~a_CAN_DELAY_INJECT;
             }else{
-               if(a_coll_edit(((*cp == 'v') ? 'v' : 'e'), hp, NULL
-                     ) != su_ERR_NONE)
-                  goto jerr;
-               /* Print msg mandated by the Mail Reference Manual */
+jinject_restart:
+               if((cp = ok_vlook(editalong)) == NIL){
+                  if(msf & n_MAILSEND_HEADERS_PRINT)
+                     n_puthead(TRU1, hp, n_stdout, gfield, SEND_TODISP,
+                        CONV_NONE, NIL, NIL);
+
+                  rewind(_coll_fp);
+                  while((c = getc(_coll_fp)) != EOF) /* XXX bytewise, yuck! */
+                     putc(c, n_stdout);
+                  if(fseek(_coll_fp, 0, SEEK_END))
+                     goto jerr;
+                  fflush(n_stdout);
+               }else{
+                  if(a_coll_edit(((*cp == 'v') ? 'v' : 'e'), hp, NIL
+                        ) != su_ERR_NONE)
+                     goto jerr;
+
+                  /* Print msg mandated by the Mail Reference Manual */
 jcont:
-               if((n_psonce & n_PSO_INTERACTIVE) && !(n_pstate & n_PS_ROBOT))
-                  fputs(_("(continue)\n"), n_stdout);
+                  if((n_psonce & n_PSO_INTERACTIVE) &&
+                        !(n_pstate & n_PS_ROBOT) &&
+                        !(flags & a_NEED_INJECT_RESTART))
+                     fputs(_("(continue)\n"), n_stdout);
+                  fflush(n_stdout);
+               }
             }
-            fflush(n_stdout);
          }
       }
    } else {
@@ -1179,7 +1216,7 @@ jcont:
    }
 
    /* If not under shell hook control */
-   if(coap == NULL){
+   if(coap == NIL){
       /* We're done with -M or -m TODO because: we are too stupid yet, above */
       if(n_poption & n_PO_Mm_FLAG)
          goto jout;
@@ -1189,6 +1226,7 @@ jcont:
          /* Need to go over n_go_input() to handle injections nonetheless */
          enum n_go_input_flags gif;
 
+         ASSERT(!(flags & a_NEED_INJECT_RESTART));
          for(gif = n_GO_INPUT_CTX_COMPOSE | n_GO_INPUT_DELAY_INJECTIONS;;){
             cnt = n_go_input(gif, n_empty, &linebuf, &linesize, NULL, NULL);
             if(cnt < 0){
@@ -1200,7 +1238,8 @@ jcont:
                }
                break;
             }
-            i = (uz)cnt;
+
+            i = S(uz,cnt);
             if(i != fwrite(linebuf, sizeof *linebuf, i, _coll_fp))
                goto jerr;
             /* TODO n_PS_READLINE_NL is a hack to ensure that _in_all_-
@@ -1214,12 +1253,14 @@ jcont:
          }
          goto jout;
       }
+
+      escape = *ok_vlook(escape);
    }
 
-   /* The interactive collect loop */
-   if(coap == NULL)
-      escape = *ok_vlook(escape);
-   flags = ok_blook(errexit) ? a_ERREXIT : a_NONE;
+   /* The "interactive" collect loop */
+   flags &= a_ROUND_MASK;
+   if(ok_blook(errexit))
+      flags |= a_ERREXIT;
 
    for(;;){
       enum {a_HIST_NONE, a_HIST_ADD = 1u<<0, a_HIST_GABBY = 1u<<1} hist;
@@ -1229,27 +1270,51 @@ jcont:
          boole histadd;
 
          /* TODO optimize: no need to evaluate that anew for each loop tick! */
+         histadd = (s != NIL);
          gif = n_GO_INPUT_CTX_COMPOSE;
-         histadd = (s != NULL);
-         if(!(n_poption & n_PO_t_FLAG) || (n_psonce & n_PSO_t_FLAG_DONE)){
-            if((n_psonce & n_PSO_INTERACTIVE) || (n_poption & n_PO_TILDE_FLAG))
-               gif |= n_GO_INPUT_NL_ESC;
-         }else
+         if(flags & a_CAN_DELAY_INJECT)
             gif |= n_GO_INPUT_DELAY_INJECTIONS;
+
+         if((n_poption & n_PO_t_FLAG) && !(n_psonce & n_PSO_t_FLAG_DONE)){
+            ASSERT(!(flags & a_NEED_INJECT_RESTART));
+         }else{
+            if(n_psonce & n_PSO_INTERACTIVE){
+               gif |= n_GO_INPUT_NL_ESC;
+               if(UNLIKELY((flags & a_NEED_INJECT_RESTART) &&
+                     !n_go_input_have_injections())){
+                  flags &= ~a_NEED_INJECT_RESTART;
+                  goto jinject_restart;
+               }
+            }else{
+               ASSERT(!(flags & a_NEED_INJECT_RESTART));
+               if(n_poption & n_PO_TILDE_FLAG)
+                  gif |= n_GO_INPUT_NL_ESC;
+            }
+         }
+
          cnt = n_go_input(gif, n_empty, &linebuf, &linesize, NULL, &histadd);
          hist = histadd ? a_HIST_ADD | a_HIST_GABBY : a_HIST_NONE;
       }
 
       if(cnt < 0){ /* TODO n_go_input_is_eof()!  Could be error!! */
-         if(coap != NULL)
+         if(coap != NIL)
             break;
+
          if((n_poption & n_PO_t_FLAG) && !(n_psonce & n_PSO_t_FLAG_DONE)){
             fflush_rewind(_coll_fp);
             n_psonce |= n_PSO_t_FLAG_DONE;
+            flags &= ~a_CAN_DELAY_INJECT;
             if(!a_coll_makeheader(_coll_fp, hp, checkaddr_err, TRU1))
                goto jerr;
             continue;
-         }else if((n_psonce & n_PSO_INTERACTIVE) && !(n_pstate & n_PS_ROBOT) &&
+         }
+
+         if((flags & a_CAN_DELAY_INJECT) && n_go_input_have_injections()){
+            flags &= ~a_CAN_DELAY_INJECT;
+            continue;
+         }
+
+         if((n_psonce & n_PSO_INTERACTIVE) && !(n_pstate & n_PS_ROBOT) &&
                ok_blook(ignoreeof) && ++eofcnt < 4){
             fprintf(n_stdout,
                _("*ignoreeof* set, use `~.' to terminate letter\n"));
@@ -1403,7 +1468,7 @@ jearg:
 "~.            Commit and send message\n"
 "~: <command>  Execute an internal command\n"
 "~< <file>     Insert <file> (\"~<! <command>\": insert shell command)\n"
-"~@ [<files>]  Edit [Add] attachments (file[=in-charset[#out-charset]])\n"
+"~@ [<files>]  Edit [Add] attachments (file[=in-charset[#out-charset]], #no)\n"
 "~c <users>    Add users to Cc: list (`~b': to Bcc:)\n"
 "~e, ~v        Edit message via $EDITOR / $VISUAL\n"
             ), n_stdout);
@@ -1448,7 +1513,7 @@ jearg:
          n_pstate_ex_no = 0; /* XXX */
          }break;
       case '^':
-         if(!n_dig_msg_circumflex(&dmc, n_stdout, cp)){
+         if(!mx_dig_msg_circumflex(&dmc, n_stdout, cp)){
             if(ferror(_coll_fp))
                goto jerr;
             goto jearg;
@@ -1775,7 +1840,7 @@ jhistcont:
          /* Do not add *escape* to the history in order to allow history search
           * to be handled generically in the MLE regardless of actual *escape*
           * settings etc. */
-         n_tty_addhist(&n_string_cp(s)[1], (n_GO_INPUT_CTX_COMPOSE |
+         mx_tty_addhist(&n_string_cp(s)[1], (n_GO_INPUT_CTX_COMPOSE |
             (hist & a_HIST_GABBY ? n_GO_INPUT_HIST_GABBY : n_GO_INPUT_NONE)));
       }
       if(c != '\0')
@@ -1783,6 +1848,8 @@ jhistcont:
    }
 
 jout:
+   flags |= a_EVER_LEFT_INPUT_LOOPS;
+
    /* Do we have *on-compose-splice-shell*, or *on-compose-splice*?
     * TODO Usual f...ed up state of signals and terminal etc. */
    if(coap == NULL && (cp = ok_vlook(on_compose_splice_shell)) != NULL) Jocs:{
@@ -1792,11 +1859,12 @@ jout:
       /* Reset *escape* and more to their defaults.  On change update manual! */
       if(ifs_saved == NULL)
          ifs_saved = savestr(ok_vlook(ifs));
-      escape = n_ESCAPE[0];
       ok_vclear(ifs);
+      escape = n_ESCAPE[0];
+      flags &= ~a_CAN_DELAY_INJECT;
 
       if(coapm != NULL){
-         /* XXX Due Popen() fflush(NULL) in PTF mode, ensure nothing to flush */
+         /* XXX Due pipe_open() fflush(NIL) in PTF mode */
          /*if(!n_real_seek(_coll_fp, 0, SEEK_END))
           *  goto jerr;*/
          u.ptf = &a_coll_ocs__mac;
@@ -1811,28 +1879,29 @@ jout:
       coap = n_lofi_alloc(VSTRUCT_SIZEOF(struct a_coll_ocs_arg, coa_cmd
             ) + i);
       coap->coa_pipe[0] = coap->coa_pipe[1] = -1;
-      coap->coa_stdin = coap->coa_stdout = NULL;
+      coap->coa_stdin = coap->coa_stdout = NIL;
       coap->coa_senderr = checkaddr_err;
       su_mem_copy(coap->coa_cmd, cp, i);
 
-      hold_all_sigs();
+      mx_sigs_all_holdx();
       coap->coa_opipe = safe_signal(SIGPIPE, SIG_IGN);
       coap->coa_oint = safe_signal(SIGINT, SIG_IGN);
-      rele_all_sigs();
+      mx_sigs_all_rele();
 
-      if(pipe_cloexec(coap->coa_pipe) != -1 &&
-            (coap->coa_stdin = Fdopen(coap->coa_pipe[0], "r", FAL0)) != NULL &&
-            (coap->coa_stdout = Popen(cmd, "W", u.sh, NULL, coap->coa_pipe[1])
-             ) != NULL){
-         close(coap->coa_pipe[1]);
+      if(mx_fs_pipe_cloexec(coap->coa_pipe) &&
+            (coap->coa_stdin = mx_fs_fd_open(coap->coa_pipe[0], "r", FAL0)
+               ) != NIL &&
+            (coap->coa_stdout = mx_fs_pipe_open(cmd, "W", u.sh, NIL,
+               coap->coa_pipe[1])) != NIL){
+         close(S(int,coap->coa_pipe[1]));
          coap->coa_pipe[1] = -1;
 
          temporary_compose_mode_hook_call(NULL, NULL, NULL);
          n_go_splice_hack(coap->coa_cmd, coap->coa_stdin, coap->coa_stdout,
-            (n_psonce & ~(n_PSO_INTERACTIVE | n_PSO_TTYIN | n_PSO_TTYOUT)),
+            (n_psonce & ~(n_PSO_INTERACTIVE | n_PSO_TTYANY)),
             &a_coll_ocs__finalize, &coap);
          /* Hook version protocol for ~^: update manual upon change! */
-         fputs(n_DIG_MSG_PLUMBING_VERSION "\n", n_stdout/*coap->coa_stdout*/);
+         fputs(mx_DIG_MSG_PLUMBING_VERSION "\n", n_stdout/*coap->coa_stdout*/);
          goto jcont;
       }
 
@@ -1845,11 +1914,11 @@ jout:
       *checkaddr_err = 0;
       goto jerr;
    }
-   if(coapm == NULL && (coapm = ok_vlook(on_compose_splice)) != NULL)
+   if(coapm == NIL && (coapm = ok_vlook(on_compose_splice)) != NIL)
       goto Jocs;
-   if(coap != NULL){
+   if(coap != NIL && ifs_saved != NIL){
       ok_vset(ifs, ifs_saved);
-      ifs_saved = NULL;
+      ifs_saved = NIL;
    }
 
    /*
@@ -1860,15 +1929,13 @@ jout:
    /* Final chance to edit headers, if not already above; and *asksend* */
    if((n_psonce & n_PSO_INTERACTIVE) && !(n_pstate & n_PS_ROBOT)){
       if(ok_blook(bsdcompat) || ok_blook(askatend)){
-         enum gfield gf;
-
-         gf = GNONE;
+         gfield = GNONE;
          if(ok_blook(askcc))
-            gf |= GCC;
+            gfield |= GCC;
          if(ok_blook(askbcc))
-            gf |= GBCC;
-         if(gf != 0)
-            grab_headers(n_GO_INPUT_CTX_COMPOSE, hp, gf, 1);
+            gfield |= GBCC;
+         if(gfield != GNONE)
+            grab_headers(n_GO_INPUT_CTX_COMPOSE, hp, gfield, 1);
       }
 
       if(ok_blook(askattach))
@@ -1942,7 +2009,7 @@ jreasksend:
       }
       cpq = n_shexp_quote_cp(cp = cpq, FAL0);
 
-      if((sigfp = Fopen(cp, "r")) == NULL){
+      if((sigfp = mx_fs_open(cp, "r")) == NIL){
          n_err(_("Can't open *signature* %s: %s\n"),
             cpq, su_err_doc(su_err_no()));
          goto jerr;
@@ -1963,7 +2030,7 @@ jreasksend:
          int e = su_err_no(), ise = ferror(x);
 
          sigfp = NULL;
-         Fclose(x);
+         mx_fs_close(x);
 
          if(ise){
             n_err(_("Errors while reading *signature* %s: %s\n"),
@@ -2008,10 +2075,9 @@ jskiptails:
    }
 
 jleave:
-   if (linebuf != NULL)
-      n_free(linebuf);
    sigprocmask(SIG_BLOCK, &nset, NULL);
-   n_DIG_MSG_COMPOSE_GUT(&dmc);
+   mx_fs_linepool_release(linebuf, linesize);
+   mx_DIG_MSG_COMPOSE_GUT(&dmc);
    n_pstate &= ~n_PS_COMPOSE_MODE;
    safe_signal(SIGINT, _coll_saveint);
    safe_signal(SIGHUP, _coll_savehup);
@@ -2020,11 +2086,11 @@ jleave:
    return _coll_fp;
 
 jerr:
-   hold_all_sigs();
+   mx_sigs_all_holdx();
 
    if(coap != NULL && coap != (struct a_coll_ocs_arg*)-1){
       if(!(flags & a_COAP_NOSIGTERM))
-         n_psignal(coap->coa_stdout, SIGTERM);
+         mx_fs_pipe_signal(coap->coa_stdout, SIGTERM);
       n_go_splice_hack_remove_after_jump();
       coap = NULL;
    }
@@ -2032,16 +2098,16 @@ jerr:
       ok_vset(ifs, ifs_saved);
       ifs_saved = NULL;
    }
-   if(sigfp != NULL){
-      Fclose(UNVOLATILE(FILE*,sigfp));
+   if(sigfp != NIL){
+      mx_fs_close(UNVOLATILE(FILE*,sigfp));
       sigfp = NULL;
    }
-   if(_coll_fp != NULL){
-      Fclose(_coll_fp);
-      _coll_fp = NULL;
+   if(_coll_fp != NIL){
+      mx_fs_close(_coll_fp);
+      _coll_fp = NIL;
    }
 
-   rele_all_sigs();
+   mx_sigs_all_rele();
 
    ASSERT(checkaddr_err != NULL);
    /* TODO We don't save in $DEAD upon error because msg not readily composed?
