@@ -44,8 +44,15 @@
 #include <pwd.h>
 
 #include <su/cs.h>
+#include <su/mem.h>
 
-#include "mx/shortcut.h"
+#include "mx/cmd-shortcut.h"
+#include "mx/dig-msg.h"
+#include "mx/file-locks.h"
+#include "mx/file-streams.h"
+#include "mx/net-pop3.h"
+#include "mx/net-socket.h"
+#include "mx/sigs.h"
 #include "mx/ui-str.h"
 
 /* TODO fake */
@@ -246,18 +253,19 @@ a_folder_mbox_setptr(FILE *ibuf, off_t offset){ /* TODO Mailbox->setptr() */
    uz filesize, linesize, cnt;
    NYD_IN;
 
+   filesize = mailsize - offset;
+
    su_mem_set(&self, 0, sizeof self);
    self.m_flag = MUSED | MNEW | MNEWEST;
-   filesize = mailsize - offset;
+
    offset = ftell(mb.mb_otf);
    f = a_MAYBE | (ok_blook(mbox_rfc4155) ? a_RFC4155 : 0);
 
-   linebuf = NULL, linesize = 0; /* TODO string pool */
-
+   mx_fs_linepool_aquire(&linebuf, &linesize);
    for(;;){
       /* Ensure space for terminating LF, so do append it */
       if(UNLIKELY(fgetline(&linebuf, &linesize, &filesize, &cnt, ibuf, TRU1
-            ) == NULL)){
+            ) == NIL)){
          if(f & a_HADONE){
             if(f & a_CREATE){
                commit.m_size += self.m_size;
@@ -272,7 +280,7 @@ a_folder_mbox_setptr(FILE *ibuf, off_t offset){ /* TODO Mailbox->setptr() */
          message_append_null();
 
          if(f & a_HAD_BAD_FROM_){
-            if(!(mb.mb_active & MB_BAD_FROM_)){
+            /*if(!(mb.mb_active & MB_BAD_FROM_))*/{
                mb.mb_active |= MB_BAD_FROM_;
                /* TODO mbox-rfc4155 does NOT fix From_ line! */
                n_err(_("MBOX contains non-conforming From_ line(s)!\n"
@@ -280,13 +288,12 @@ a_folder_mbox_setptr(FILE *ibuf, off_t offset){ /* TODO Mailbox->setptr() */
                   "  Setting *mbox-rfc4155* and reopening _may_ "
                      "improve the result.\n"
                   "  Recreating the mailbox will perform MBOXO quoting: "
-                     "\"copy * SOME-FILE\".  "
-                     "(Then unset *mbox-rfc4155* again.)\n"));
+                     "\"copy * SOME-FILE\".\n"
+                  "  (Then unset *mbox-rfc4155* again.)\n"));
             }
          }
 
-         if(linebuf != NULL)
-            n_free(linebuf);
+         mx_fs_linepool_release(linebuf, linesize);
          break;
       }
 
@@ -425,6 +432,7 @@ jputln:
       linebuf[cnt++] = '\n';
       ASSERT(linebuf[cnt] == '\0');
       fwrite(linebuf, sizeof *linebuf, cnt, mb.mb_otf);
+
       if(ferror(mb.mb_otf)){
          n_perr(_("/tmp"), 0);
          exit(n_EXIT_ERR); /* TODO no! */
@@ -520,7 +528,7 @@ jlogname:
 #ifdef mx_HAVE_POP3
    case PROTO_POP3:
       shudclob = 1;
-      rv = pop3_setfile(who, orig_name, fm);
+      rv = mx_pop3_setfile(who, orig_name, fm);
       goto jleave;
 #endif
 #ifdef mx_HAVE_IMAP
@@ -537,7 +545,7 @@ jlogname:
       goto jem1;
    }
 
-   if ((ibuf = n_fopen_any(savecat("file://", name), "r", NULL)) == NULL) {
+   if((ibuf = mx_fs_open_any(savecat("file://", name), "r", NIL)) == NIL){
       int e = su_err_no();
 
       if ((fm & FEDIT_SYSBOX) && e == su_ERR_NOENT) {
@@ -554,8 +562,8 @@ jlogname:
       if (fm & FEDIT_NEWMAIL)
          goto jleave;
 
-      if(mb.mb_digmsg != NULL)
-         n_dig_msg_on_mailbox_close(&mb);
+      if(mb.mb_digmsg != NIL)
+         mx_dig_msg_on_mailbox_close(&mb);
       mb.mb_type = MB_VOID;
 
       if (ok_blook(emptystart)) {
@@ -591,9 +599,13 @@ jlogname:
 
    hold_sigs();
 
-#ifdef mx_HAVE_SOCKETS
-   if (!(fm & FEDIT_NEWMAIL) && mb.mb_sock.s_fd >= 0)
-      sclose(&mb.mb_sock); /* TODO sorry? VMAILFS->close(), thank you */
+#ifdef mx_HAVE_NET
+   if(!(fm & FEDIT_NEWMAIL) && mb.mb_sock != NIL){
+      if(mb.mb_sock->s_fd >= 0)
+         mx_socket_close(mb.mb_sock); /* TODO VMAILFS->close() on open thing */
+      su_FREE(mb.mb_sock);
+      mb.mb_sock = NIL;
+   }
 #endif
 
    /* TODO There is no intermediate VOID box we've switched to: name may
@@ -604,9 +616,9 @@ jlogname:
     * TODO modified so we can't tell: Mailbox::is_modified() :-(( */
    if (/*shudclob && !(fm & FEDIT_NEWMAIL) &&*/ !su_cs_cmp(name, mailname)) {
       name = mailname;
-      Fclose(ibuf);
+      mx_fs_close(ibuf);
 
-      if ((ibuf = n_fopen_any(name, "r", NULL)) == NULL ||
+      if((ibuf = mx_fs_open_any(name, "r", NIL)) == NIL ||
             fstat(fileno(ibuf), &stb) == -1 ||
             (!S_ISREG(stb.st_mode) && !isdevnull)) {
          n_perr(name, 0);
@@ -652,10 +664,10 @@ jlogname:
 
    if (isdevnull)
       lckfp = (FILE*)-1;
-   else if (!(n_pstate & n_PS_EDIT))
-      lckfp = n_dotlock(name, fileno(ibuf), FLT_READ, offset,0,
-            (fm & FEDIT_NEWMAIL ? 0 : UZ_MAX));
-   else if (n_file_lock(fileno(ibuf), FLT_READ, offset,0,
+   else if(!(n_pstate & n_PS_EDIT))
+      lckfp = mx_file_dotlock(name, fileno(ibuf), mx_FILE_LOCK_TYPE_READ,
+            offset,0, (fm & FEDIT_NEWMAIL ? 0 : UZ_MAX));
+   else if(mx_file_lock(fileno(ibuf), mx_FILE_LOCK_TYPE_READ, offset,0,
          (fm & FEDIT_NEWMAIL ? 0 : UZ_MAX)))
       lckfp = (FILE*)-1;
 
@@ -689,11 +701,11 @@ jlogname:
       c_sort((void*)-1);
    }
 
-   Fclose(ibuf);
-   ibuf = NULL;
-   if (lckfp != NULL && lckfp != (FILE*)-1) {
-      Pclose(lckfp, FAL0);
-      /*lckfp = NULL;*/
+   mx_fs_close(ibuf);
+   ibuf = NIL;
+   if(lckfp != NIL && lckfp != R(FILE*,-1)){
+      mx_fs_pipe_close(lckfp, FAL0);
+      /*lckfp = NIL;*/
    }
 
    if (!(fm & FEDIT_NEWMAIL)) {
@@ -724,16 +736,18 @@ jlogname:
    if(fm & FEDIT_NEWMAIL)
       newmailinfo(omsgCount);
 jleave:
-   if (ibuf != NULL) {
-      Fclose(ibuf);
-      if (lckfp != NULL && lckfp != (FILE*)-1)
-         Pclose(lckfp, FAL0);
+   if(ibuf != NIL){
+      mx_fs_close(ibuf);
+      if(lckfp != NIL && lckfp != R(FILE*,-1))
+         mx_fs_pipe_close(lckfp, FAL0);
    }
+
    NYD_OU;
    return rv;
+
 jem2:
-   if(mb.mb_digmsg != NULL)
-      n_dig_msg_on_mailbox_close(&mb);
+   if(mb.mb_digmsg != NIL)
+      mx_dig_msg_on_mailbox_close(&mb);
    mb.mb_type = MB_VOID;
 jem1:
    su_err_set_no(su_ERR_NOTOBACCO);
@@ -804,7 +818,7 @@ print_header_summary(char const *Larg)
 
    if (Larg != NULL) {
       /* Avoid any messages XXX add a make_mua_silent() and use it? */
-      if ((n_poption & (n_PO_VERB | n_PO_EXISTONLY)) == n_PO_EXISTONLY) {
+      if ((n_poption & (n_PO_V | n_PO_EXISTONLY)) == n_PO_EXISTONLY) {
          n_stdout = freopen(n_path_devnull, "w", stdout);
          n_stderr = freopen(n_path_devnull, "w", stderr);
       }
@@ -966,8 +980,8 @@ jleave:
 FL void
 initbox(char const *name)
 {
+   struct mx_fs_tmp_ctx *fstcp;
    boole err;
-   char *tempMesg;
    NYD_IN;
 
    if (mb.mb_type != MB_VOID)
@@ -978,15 +992,16 @@ initbox(char const *name)
    _update_mailname((name != mailname) ? name : NULL);
 
    err = FAL0;
-   if((mb.mb_otf = Ftmp(&tempMesg, "tmpmbox", OF_WRONLY | OF_HOLDSIGS)) ==
-         NULL){
+   if((mb.mb_otf = mx_fs_tmp_open("tmpmbox", (mx_FS_O_WRONLY |
+            mx_FS_O_HOLDSIGS), &fstcp)) == NIL){
       n_perr(_("initbox: temporary mail message file, writer"), 0);
       err = TRU1;
-   }else if((mb.mb_itf = safe_fopen(tempMesg, "r", NULL)) == NULL) {
+   }else if((mb.mb_itf = mx_fs_open(fstcp->fstc_filename, "&r")
+         ) == NIL){
       n_perr(_("initbox: temporary mail message file, reader"), 0);
       err = TRU1;
    }
-   Ftmp_release(&tempMesg);
+   mx_fs_tmp_release(fstcp);
    if(err)
       exit(n_EXIT_ERR);
 
@@ -1153,7 +1168,7 @@ jset:
 }
 
 FL int
-n_folder_mbox_prepare_append(FILE *fout, struct stat *st_or_null){
+n_folder_mbox_prepare_append(FILE *fout, struct stat *st_or_nil){
    /* TODO n_folder_mbox_prepare_append -> Mailbox->append() */
    struct stat stb;
    char buf[2];
@@ -1161,22 +1176,20 @@ n_folder_mbox_prepare_append(FILE *fout, struct stat *st_or_null){
    boole needsep;
    NYD2_IN;
 
-   if(!fseek(fout, -2L, SEEK_END)){
-      if(fread(buf, sizeof *buf, 2, fout) != 2)
-         goto jerrno;
+   if(!fseek(fout, -2L, SEEK_END) && fread(buf, sizeof *buf, 2, fout) == 2)
       needsep = (buf[0] != '\n' || buf[1] != '\n');
-   }else{
+   else{
       rv = su_err_no();
-
-      if(st_or_null == NULL){
-         st_or_null = &stb;
-         if(fstat(fileno(fout), st_or_null))
+      if(st_or_nil == NIL){
+         st_or_nil = &stb;
+         if(fstat(fileno(fout), st_or_nil))
             goto jerrno;
       }
 
-      if(st_or_null->st_size >= 2)
+      if(st_or_nil->st_size >= 2)
          goto jleave;
-      if(st_or_null->st_size == 0){
+      if(st_or_nil->st_size == 0){
+         clearerr(fout);
          rv = su_ERR_NONE;
          goto jleave;
       }
