@@ -53,8 +53,12 @@ su_EMPTY_FILE()
 
 #include <su/cs.h>
 #include <su/icodec.h>
+#include <su/mem.h>
 #include <su/prime.h>
 
+#include "mx/file-streams.h"
+#include "mx/sigs.h"
+
 /* TODO fake */
 #include "su/code-in.h"
 
@@ -69,8 +73,8 @@ static void             __maildircatch_hold(int s);
 /* Do some cleanup in the tmp/ subdir */
 static void             _cleantmp(void);
 
-static int              _maildir_setfile1(char const *name, enum fedit_mode fm,
-                           int omsgCount);
+static int a_maildir_setfile1(char const *name, enum fedit_mode fm,
+      int omsgCount);
 
 static int a_maildir_cmp(void const *a, void const *b);
 
@@ -80,7 +84,7 @@ static int              _maildir_subdir(char const *name, char const *sub,
 static void             _maildir_append(char const *name, char const *sub,
                            char const *fn);
 
-static void             readin(char const *name, struct message *m);
+static boole a_maildir_readin(char const *name, struct message *mp);
 
 static void             maildir_update(void);
 
@@ -136,7 +140,7 @@ _cleantmp(void)
    if ((dirp = opendir("tmp")) == NULL)
       goto jleave;
 
-   now = n_time_now(FAL0)->ts_sec;
+   now = n_time_now(FAL0)->ts_sec - 36*3600;
    s = n_string_creat_auto(&s_b);
 
    while ((dp = readdir(dirp)) != NULL) {
@@ -148,7 +152,7 @@ _cleantmp(void)
       s = n_string_push_cp(s, dp->d_name);
       if (stat(n_string_cp(s), &st) == -1)
          continue;
-      if (st.st_atime + 36*3600 < now)
+      if (st.st_atime <= now)
          unlink(s->s_dat);
    }
    closedir(dirp);
@@ -157,7 +161,7 @@ jleave:
 }
 
 static int
-_maildir_setfile1(char const *name, enum fedit_mode fm, int omsgCount)
+a_maildir_setfile1(char const *name, enum fedit_mode fm, int omsgCount)
 {
    int i;
    NYD_IN;
@@ -174,11 +178,16 @@ _maildir_setfile1(char const *name, enum fedit_mode fm, int omsgCount)
    _maildir_append(name, NULL, NULL);
 
    n_autorec_relax_create();
-   for (i = ((fm & FEDIT_NEWMAIL) ? omsgCount : 0); i < msgCount; ++i) {
-      readin(name, message + i);
+   for(i = ((fm & FEDIT_NEWMAIL) ? omsgCount : 0); i < msgCount; ++i){
+      if(!a_maildir_readin(name, &message[i])){
+         i = -1;
+         break;
+      }
       n_autorec_relax_unroll();
    }
    n_autorec_relax_gut();
+   if(i < 0)
+      goto jleave;
 
    if (fm & FEDIT_NEWMAIL) {
       if (msgCount > omsgCount)
@@ -449,44 +458,44 @@ jleave:
    return;
 }
 
-static void
-readin(char const *name, struct message *m)
-{
+static boole
+a_maildir_readin(char const *name, struct message *mp){
+   long size, lines;
+   boole b;
    char *buf;
-   uz bufsize, buflen, cnt;
-   long size = 0, lines = 0;
+   uz cnt, buflen, bufsize;
    off_t offset;
    FILE *fp;
-   int emptyline = 0;
+   char const *emsg;
    NYD_IN;
 
-   if ((fp = Fopen(m->m_maildir_file, "r")) == NULL) {
-      n_err(_("Cannot read %s for message %lu\n"),
-         n_shexp_quote_cp(savecatsep(name, '/', m->m_maildir_file), FAL0),
-         (ul)P2UZ(m - message + 1));
-      m->m_flag |= MHIDDEN;
-      goto jleave;
+   if((fp = mx_fs_open(mp->m_maildir_file, "r")) == NIL){
+      emsg = _("Cannot read %s for message %lu\n");
+      goto jerr;
    }
 
-   cnt = fsize(fp);
-   fseek(mb.mb_otf, 0L, SEEK_END);
    offset = ftell(mb.mb_otf);
-   buf = n_alloc(bufsize = LINESIZE);
+   cnt = fsize(fp);
+
+   mx_fs_linepool_aquire(&buf, &bufsize);
    buflen = 0;
-   while (fgetline(&buf, &bufsize, &cnt, &buflen, fp, 1) != NULL) {
+   b = FAL0;
+   size = lines = 0;
+   while(fgetline(&buf, &bufsize, &cnt, &buflen, fp, 1) != NIL){
       /* Since we simply copy over data without doing any transfer
        * encoding reclassification/adjustment we *have* to perform
        * RFC 4155 compliant From_ quoting here */
-      if (emptyline && is_head(buf, buflen, FAL0)) {
+      if(b && is_head(buf, buflen, FAL0)){
          putc('>', mb.mb_otf);
          ++size;
       }
-      size += fwrite(buf, 1, buflen, mb.mb_otf);/*XXX err hdling*/
-      emptyline = (*buf == '\n');
+      size += fwrite(buf, 1, buflen, mb.mb_otf);
+      b = (*buf == '\n');
       ++lines;
    }
-   n_free(buf);
-   if (!emptyline) {
+   mx_fs_linepool_release(buf, bufsize);
+
+   if(!b){
       /* TODO we need \n\n for mbox format.
        * TODO That is to say we do it wrong here in order to get it right
        * TODO when send.c stuff or with MBOX handling, even though THIS
@@ -495,16 +504,30 @@ readin(char const *name, struct message *m)
       ++lines;
       ++size;
    }
+   b = (fflush(mb.mb_otf) != EOF);
 
-   Fclose(fp);
-   fflush(mb.mb_otf);
-   m->m_size = m->m_xsize = size;
-   m->m_lines = m->m_xlines = lines;
-   m->m_block = mailx_blockof(offset);
-   m->m_offset = mailx_offsetof(offset);
-   substdate(m);
+   mx_fs_close(fp);
+
+   if(!b){
+      emsg = _("I/O error reading %s for message %lu\n");
+      goto jleave;
+   }
+
+   mp->m_size = mp->m_xsize = size;
+   mp->m_lines = mp->m_xlines = lines;
+   mp->m_block = mailx_blockof(offset);
+   mp->m_offset = mailx_offsetof(offset);
+   substdate(mp);
 jleave:
    NYD_OU;
+   return b;
+
+jerr:
+   n_err(emsg,
+      n_shexp_quote_cp(savecatsep(name, '/', mp->m_maildir_file), FAL0),
+      S(ul,P2UZ(mp - message + 1)));
+   b = FAL0;
+   goto jleave;
 }
 
 static void
@@ -711,8 +734,8 @@ maildir_append1(struct n_timespec const *tsp, char const *name, FILE *fp,
       snprintf(tfn, n, "%s/tmp/%s", name, fn);
 
       /* Use "wx" for O_EXCL XXX stat(2) rather redundant; coverity:TOCTOU */
-      if ((!stat(tfn, &st) || su_err_no() == su_ERR_NOENT) &&
-            (op = Fopen(tfn, "wx")) != NULL)
+      if((!stat(tfn, &st) || su_err_no() == su_ERR_NOENT) &&
+            (op = mx_fs_open(tfn, "wx")) != NIL)
          break;
 
       nfn = (char*)(P2UZ(nfn) - 1);
@@ -731,12 +754,12 @@ maildir_append1(struct n_timespec const *tsp, char const *name, FILE *fp,
       if (z != (n = fread(buf, 1, z, fp)) || n != fwrite(buf, 1, n, op)) {
 jtmperr:
          n_err(_("Error writing to %s\n"), n_shexp_quote_cp(tfn, FAL0));
-         Fclose(op);
+         mx_fs_close(op);
          goto jerr;
       }
       size -= n;
    }
-   Fclose(op);
+   mx_fs_close(op);
 
    nfn = n_autorec_alloc(n = nlen + flen + 6);
    snprintf(nfn, n, "%s/new/%s", name, fn);
@@ -971,19 +994,18 @@ jerr:
          mktable();
       if (saveint != SIG_IGN)
          safe_signal(SIGINT, &__maildircatch);
-      i = _maildir_setfile1(name, fm, omsgCount);
+      if(a_maildir_setfile1(name, fm, omsgCount) < 0){
+         if((fm & FEDIT_NEWMAIL) && a_maildir_tbl != NIL)
+            n_free(a_maildir_tbl);
+         emsg = N_("Cannot setup maildir://%s\n");
+         goto jerr;
+      }
    }
    if ((fm & FEDIT_NEWMAIL) && a_maildir_tbl != NULL)
       n_free(a_maildir_tbl);
 
    safe_signal(SIGINT, saveint);
 
-   if (i < 0) {
-      mb.mb_type = MB_VOID;
-      *mailname = '\0';
-      msgCount = 0;
-   }
-
    if (cwret(&cw) == STOP)
       n_panic(_("Cannot change back to current directory"));
    cwrelse(&cw);
@@ -1082,7 +1104,7 @@ maildir_append(char const *name, FILE *fp, long offset)
    if ((rv = mkmaildir(name)) != OKAY)
       goto jleave;
 
-   buf = n_alloc(bufsize = LINESIZE); /* TODO line pool; signals */
+   mx_fs_linepool_aquire(&buf, &bufsize);
    buflen = 0;
    cnt = fsize(fp);
    offs = offset /* BSD will move due to O_APPEND! ftell(fp) */;
@@ -1156,10 +1178,11 @@ maildir_append(char const *name, FILE *fp, long offset)
          }
       }
    }
+
    ASSERT(rv == OKAY);
 jfree:
    n_autorec_relax_gut();
-   n_free(buf);
+   mx_fs_linepool_release(buf, bufsize);
 jleave:
    NYD_OU;
    return rv;
