@@ -26,6 +26,9 @@
 
 #include <su/cs.h>
 #include <su/icodec.h>
+#include <su/mem.h>
+
+#include "mx/random.h"
 
 #include "mx/iconv.h"
 
@@ -356,7 +359,7 @@ jeeqaaster:
    ASSERT(head != NULL); /* (always true due to jumpin:, but..) */
 
    errors |= __rfc2231_join(head, &rv, &emsg);
-   if (errors && (n_poption & n_PO_D_V)) {
+   if (errors /*&& (n_poption & n_PO_D_V)*/) {
       /* TODO should set global flags so that at the end of an operation
        * TODO (for a message) a summary can be printed: faulty MIME, xy */
       if (emsg == NULL)
@@ -372,7 +375,7 @@ jerr:
       head = np->rj_next;
       n_free(np);
    }
-   if (n_poption & n_PO_D_V) {
+   /*if (n_poption & n_PO_D_V)*/ {
       if (emsg == NULL)
          emsg = N_("expected asterisk *");
       n_err(_("Faulty RFC 2231 MIME parameter value: %s: %s\n"
@@ -667,7 +670,7 @@ jleave:
    /* Need to recurse, take care not to excess magical limit of 999 levels */
 jrecurse:
    if (self->mpb_level == 999) {
-      if (n_poption & n_PO_D_V)
+      /*if (n_poption & n_PO_D_V)*/
          n_err(_("Message RFC 2231 parameters nested too deeply!\n"));
       goto jleave;
    }
@@ -937,7 +940,7 @@ mime_param_boundary_create(void)
    bp = n_autorec_alloc(36 + 6 +1);
    bp[0] = bp[2] = bp[39] = bp[41] = '=';
    bp[1] = bp[40] = '-';
-   su_mem_copy(bp + 3, n_random_create_cp(36, &reprocnt), 36);
+   su_mem_copy(bp + 3, mx_random_create_cp(36, &reprocnt), 36);
    bp[42] = '\0';
    NYD_OU;
    return bp;
