@@ -45,9 +45,11 @@
 
 #include <su/cs.h>
 #include <su/cs-dict.h>
+#include <su/mem.h>
 #include <su/sort.h>
 
 #include "mx/iconv.h"
+#include "mx/mta-aliases.h"
 
 #include "mx/names.h"
 #include "su/code-in.h"
@@ -66,8 +68,8 @@
       su_CS_DICT_AUTO_SHRINK | su_CS_DICT_ERR_PASS)
 #define a_NM_A8S_TRESHOLD_SHIFT 2
 
-struct su_cs_dict *a_nm_alias_dp, a_nm_alias__d; /* XXX atexit gut() (DVL()) */
-struct su_cs_dict *a_nm_a8s_dp, a_nm_a8s__d; /* XXX atexit _gut() (DVL()) */
+static struct su_cs_dict *a_nm_alias_dp, a_nm_alias__d; /* XXX atexit gut()..*/
+static struct su_cs_dict *a_nm_a8s_dp, a_nm_a8s__d; /* XXX .. (DVL()) */
 
 /* Same name, while taking care for *allnet*? */
 static boole a_nm_is_same_name(char const *n1, char const *n2);
@@ -238,7 +240,7 @@ a_nm_extract1(char const *line, enum gfield ntype, char const *seps,
       nbuf = n_lofi_alloc(su_cs_len(cp = line) +1);
 
       for(tailp = headp;
-            ((cp = a_nm_yankname(cp, nbuf, seps, keepcomms)) != NULL);)
+            ((cp = a_nm_yankname(cp, nbuf, seps, keepcomms)) != NULL);){
          /* TODO Cannot set GNULL_OK because otherwise this software blows up.
           * TODO We will need a completely new way of reporting the errors of
           * TODO is_addr_invalid() ... */
@@ -249,6 +251,7 @@ a_nm_extract1(char const *line, enum gfield ntype, char const *seps,
                headp = np;
             tailp = np;
          }
+      }
 
       n_lofi_free(nbuf);
    }
@@ -284,9 +287,11 @@ a_nm_alias_expand(uz level, struct mx_name *nlist, char const *name, int ntype,
    ASSERT_NYD(a_nm_alias_dp != NIL);
    ASSERT(mx_alias_is_valid_name(name));
 
-   if(UCMP(z, level++, ==, n_ALIAS_MAXEXP)){
+   if(UCMP(z, level++, ==, n_ALIAS_MAXEXP)){ /* TODO not a real error!! */
       n_err(_("alias: stopping recursion at depth %d\n"), n_ALIAS_MAXEXP);
-      goto jleave;
+      slp_next = NIL;
+      ccp = name;
+      goto jlinkin;
    }
 
    slp_next = slp_base =
@@ -331,7 +336,6 @@ jlinkin:
       }
    }while((slp = slp_next) != NIL);
 
-jleave:
    NYD2_OU;
    return nlist;
 }
@@ -387,7 +391,7 @@ a_nm_a8s_dump(char const *cmdname, char const *key, void const *dat){
    return slp;
 }
 
-FL struct mx_name *
+struct mx_name *
 nalloc(char const *str, enum gfield ntype)
 {
    struct n_addrguts ag;
@@ -508,7 +512,7 @@ jleave:
    return np;
 }
 
-FL struct mx_name *
+struct mx_name *
 nalloc_fcc(char const *file){
    struct mx_name *nnp;
    NYD_IN;
@@ -524,7 +528,7 @@ nalloc_fcc(char const *file){
    return nnp;
 }
 
-FL struct mx_name *
+struct mx_name *
 ndup(struct mx_name *np, enum gfield ntype)
 {
    struct mx_name *nnp;
@@ -555,7 +559,7 @@ jleave:
    return nnp;
 }
 
-FL struct mx_name *
+struct mx_name *
 cat(struct mx_name *n1, struct mx_name *n2){
    struct mx_name *tail;
    NYD_IN;
@@ -579,7 +583,7 @@ jleave:
    return tail;
 }
 
-FL struct mx_name *
+struct mx_name *
 n_namelist_dup(struct mx_name const *np, enum gfield ntype){
    struct mx_name *nlist, *xnp;
    NYD_IN;
@@ -600,7 +604,7 @@ n_namelist_dup(struct mx_name const *np, enum gfield ntype){
    return nlist;
 }
 
-FL u32
+u32
 count(struct mx_name const *np){
    u32 c;
    NYD_IN;
@@ -612,7 +616,7 @@ count(struct mx_name const *np){
    return c;
 }
 
-FL u32
+u32
 count_nonlocal(struct mx_name const *np){
    u32 c;
    NYD_IN;
@@ -625,7 +629,7 @@ count_nonlocal(struct mx_name const *np){
    return c;
 }
 
-FL struct mx_name *
+struct mx_name *
 extract(char const *line, enum gfield ntype)
 {
    struct mx_name *rv;
@@ -636,7 +640,7 @@ extract(char const *line, enum gfield ntype)
    return rv;
 }
 
-FL struct mx_name *
+struct mx_name *
 lextract(char const *line, enum gfield ntype)
 {
    char *cp;
@@ -678,7 +682,7 @@ lextract(char const *line, enum gfield ntype)
    return rv;
 }
 
-FL struct mx_name *
+struct mx_name *
 n_extract_single(char const *line, enum gfield ntype){
    struct mx_name *rv;
    NYD_IN;
@@ -688,7 +692,7 @@ n_extract_single(char const *line, enum gfield ntype){
    return rv;
 }
 
-FL char *
+char *
 detract(struct mx_name *np, enum gfield ntype)
 {
    char *topp, *cp;
@@ -733,7 +737,7 @@ jleave:
    return topp;
 }
 
-FL struct mx_name *
+struct mx_name *
 grab_names(enum n_go_input_flags gif, char const *field, struct mx_name *np,
       int comma, enum gfield gflags)
 {
@@ -749,7 +753,7 @@ jloop:
    return np;
 }
 
-FL boole
+boole
 name_is_same_domain(struct mx_name const *n1, struct mx_name const *n2)
 {
    char const *d1, *d2;
@@ -765,7 +769,7 @@ name_is_same_domain(struct mx_name const *n1, struct mx_name const *n2)
    return rv;
 }
 
-FL struct mx_name *
+struct mx_name *
 checkaddrs(struct mx_name *np, enum expand_addr_check_mode eacm,
    s8 *set_on_error)
 {
@@ -778,7 +782,7 @@ checkaddrs(struct mx_name *np, enum expand_addr_check_mode eacm,
       if ((rv = is_addr_invalid(n, eacm)) != 0) {
          if (set_on_error != NULL)
             *set_on_error |= rv; /* don't loose -1! */
-         else if (eacm & EAF_MAYKEEP) /* TODO HACK!  See definition! */
+         if (eacm & EAF_MAYKEEP) /* TODO HACK!  See definition! */
             continue;
          if (n->n_blink)
             n->n_blink->n_flink = n->n_flink;
@@ -792,9 +796,9 @@ checkaddrs(struct mx_name *np, enum expand_addr_check_mode eacm,
    return np;
 }
 
-FL struct mx_name *
-n_namelist_vaporise_head(boole strip_alternates, struct header *hp,
-   enum expand_addr_check_mode eacm, s8 *set_on_error)
+struct mx_name *
+n_namelist_vaporise_head(struct header *hp, boole metoo,
+   boole strip_alternates, enum expand_addr_check_mode eacm, s8 *set_on_error)
 {
    /* TODO namelist_vaporise_head() is incredibly expensive and redundant */
    struct mx_name *tolist, *np, **npp;
@@ -803,10 +807,27 @@ n_namelist_vaporise_head(boole strip_alternates, struct header *hp,
    tolist = cat(hp->h_to, cat(hp->h_cc, cat(hp->h_bcc, hp->h_fcc)));
    hp->h_to = hp->h_cc = hp->h_bcc = hp->h_fcc = NULL;
 
-   tolist = usermap(tolist, strip_alternates/*metoo*/);
+   tolist = usermap(tolist, metoo);
+
+   /* MTA aliases are resolved last */
+#ifdef mx_HAVE_MTA_ALIASES
+   switch(mx_mta_aliases_expand(&tolist)){
+   case su_ERR_DESTADDRREQ:
+   case su_ERR_NONE:
+   case su_ERR_NOENT:
+      break;
+   default:
+      *set_on_error |= TRU1;
+      break;
+   }
+#endif
+
    if(strip_alternates)
       tolist = mx_alternates_remove(tolist, TRU1);
-   tolist = elide(checkaddrs(tolist, eacm, set_on_error));
+
+   tolist = elide(tolist);
+
+   tolist = checkaddrs(tolist, eacm, set_on_error);
 
    for (np = tolist; np != NULL; np = np->n_flink) {
       switch (np->n_type & (GDEL | GMASK)) {
@@ -817,11 +838,12 @@ n_namelist_vaporise_head(boole strip_alternates, struct header *hp,
       }
       *npp = cat(*npp, ndup(np, np->n_type | GFULL));
    }
+
    NYD_OU;
    return tolist;
 }
 
-FL struct mx_name *
+struct mx_name *
 usermap(struct mx_name *names, boole force_metoo){
    struct su_cs_dict_view dv;
    struct mx_name *nlist, **nlist_tail, *np, *nxtnp;
@@ -868,7 +890,7 @@ usermap(struct mx_name *names, boole force_metoo){
    return nlist;
 }
 
-FL struct mx_name *
+struct mx_name *
 elide(struct mx_name *names)
 {
    uz i, j, k;
@@ -944,7 +966,7 @@ jleave:
    return nlist;
 }
 
-FL int
+int
 c_alias(void *vp){
    struct su_cs_dict_view dv;
    union {void const *cvp; boole haskey; struct n_strlist *slp;} dat;
@@ -1037,7 +1059,7 @@ c_alias(void *vp){
          }else{
             n_err(_("alias: %s: invalid argument: %s\n"),
                key, n_shexp_quote_cp(val1, FAL0));
-            rv = 1;
+            /*rv = 1;*/
             continue;
          }
 
@@ -1066,7 +1088,7 @@ jleave:
    return rv;
 }
 
-FL int
+int
 c_unalias(void *vp){ /* XXX how about toolbox and generic unxy_dict()? */
    struct su_cs_dict_view dv;
    struct n_strlist *slp;
@@ -1111,7 +1133,7 @@ c_unalias(void *vp){ /* XXX how about toolbox and generic unxy_dict()? */
    return rv;
 }
 
-FL boole
+boole
 mx_alias_is_valid_name(char const *name){
    char c;
    char const *cp;
@@ -1123,7 +1145,8 @@ mx_alias_is_valid_name(char const *name){
        * As extensions allow high-bit bytes, exclamation mark and period:
        *    [[:alnum:]_#:@!.-]+ */
       /* TODO alias_is_valid_name(): locale dependent validity check,
-       * TODO with Unicode prefix valid UTF-8! */
+       * TODO with Unicode prefix valid UTF-8!
+       * TODO no support for trailing $ yet, as says Linux usernames! */
       if(!su_cs_is_alnum(c) && c != '_' && c != '-' &&
             c != '#' && c != ':' && c != '@' &&
             /* Extensions */
@@ -1135,7 +1158,7 @@ mx_alias_is_valid_name(char const *name){
    return rv;
 }
 
-FL int
+int
 c_alternates(void *vp){
    struct n_string s_b, *s;
    struct n_strlist *slp;
@@ -1211,7 +1234,7 @@ c_alternates(void *vp){
    return rv;
 }
 
-FL int
+int
 c_unalternates(void *vp){
    int rv;
    NYD_IN;
@@ -1221,7 +1244,7 @@ c_unalternates(void *vp){
    return rv;
 }
 
-FL struct mx_name *
+struct mx_name *
 mx_alternates_remove(struct mx_name *np, boole keep_single){
    /* XXX keep a single pointer, initial null, and immediate remove nodes
     * XXX on successful match unless keep single and that pointer null! */
@@ -1288,7 +1311,7 @@ mx_alternates_remove(struct mx_name *np, boole keep_single){
    return np;
 }
 
-FL boole
+boole
 mx_name_is_mine(char const *name){
    struct su_cs_dict_view dv;
    struct mx_name *xp;
