@@ -51,6 +51,11 @@ su_EMPTY_FILE()
 
 #include <su/cs.h>
 #include <su/icodec.h>
+#include <su/mem.h>
+
+#include "mx/file-locks.h"
+#include "mx/file-streams.h"
+#include "mx/url.h"
 
 /* TODO fake */
 #include "su/code-in.h"
@@ -104,7 +109,7 @@ encname(struct mailbox *mp, const char *name, int same, const char *box)
    int resz;
    NYD2_IN;
 
-   ename = urlxenc(name, TRU1);
+   ename = mx_url_xenc(name, TRU1);
    if (mp->mb_cache_directory && same && box == NULL) {
       res = n_autorec_alloc(resz = su_cs_len(mp->mb_cache_directory) +
             su_cs_len(ename) + 2);
@@ -116,7 +121,7 @@ encname(struct mailbox *mp, const char *name, int same, const char *box)
       if ((cachedir = ok_vlook(imap_cache)) == NULL ||
             (cachedir = fexpand(cachedir, FEXP_LOCAL | FEXP_NOPROTO)) == NULL)
          goto jleave;
-      eaccount = urlxenc(mp->mb_imap_account, TRU1);
+      eaccount = mx_url_xenc(mp->mb_imap_account, TRU1);
 
       if (box != NULL || su_cs_cmp_case(box = mp->mb_imap_mailbox, "INBOX")) {
          boole err;
@@ -124,7 +129,7 @@ encname(struct mailbox *mp, const char *name, int same, const char *box)
          box = imap_path_encode(box, &err);
          if(err)
             goto jleave;
-         box = urlxenc(box, TRU1);
+         box = mx_url_xenc(box, TRU1);
       } else
          box = "INBOX";
 
@@ -166,10 +171,10 @@ getcache1(struct mailbox *mp, struct message *m, enum needspec need,
    if (setflags == 0 && ((mp->mb_type != MB_IMAP && mp->mb_type != MB_CACHE) ||
          m->m_uid == 0))
       goto jleave;
-   if ((fp = Fopen(encuid(mp, m->m_uid), "r")) == NULL)
+   if((fp = mx_fs_open(encuid(mp, m->m_uid), "r")) == NIL)
       goto jleave;
 
-   n_file_lock(fileno(fp), FLT_READ, 0,0, 0);
+   mx_file_lock(fileno(fp), mx_FILE_LOCK_TYPE_READ, 0,0, 0);
    if (fscanf(fp, infofmt, &b, (unsigned long*)&xsize, &xflag,
          (unsigned long*)&xtime, &xlines) < 4)
       goto jfail;
@@ -259,7 +264,7 @@ jflags:
    }
    rv = OKAY;
 jfail:
-   Fclose(fp);
+   mx_fs_close(fp);
 jleave:
    NYD2_OU;
    return rv;
@@ -299,12 +304,12 @@ putcache(struct mailbox *mp, struct message *m)
       goto jleave;
    if ((oldoffset = ftell(mp->mb_itf)) < 0) /* XXX weird err hdling */
       oldoffset = 0;
-   if ((obuf = Fopen(name = encuid(mp, m->m_uid), "r+")) == NULL) {
-      if ((obuf = Fopen(name, "w")) == NULL)
+   if((obuf = mx_fs_open(name = encuid(mp, m->m_uid), "r+")) == NIL){
+      if((obuf = mx_fs_open(name, "w")) == NIL)
          goto jleave;
-      n_file_lock(fileno(obuf), FLT_WRITE, 0,0, 0); /* XXX err hdl */
-   } else {
-      n_file_lock(fileno(obuf), FLT_READ, 0,0, 0); /* XXX err hdl */
+      mx_file_lock(fileno(obuf), mx_FILE_LOCK_TYPE_WRITE, 0,0, 0); /* XXX err*/
+   }else{
+      mx_file_lock(fileno(obuf), mx_FILE_LOCK_TYPE_READ, 0,0, 0); /* XXX err */
       if (fscanf(obuf, infofmt, &ob, (unsigned long*)&osize, &oflag,
             (unsigned long*)&otime, &olines) >= 4 && ob != '\0' &&
             (ob == 'B' || (ob == 'H' && c != 'B'))) {
@@ -319,17 +324,19 @@ putcache(struct mailbox *mp, struct message *m)
                USEBITS(m->m_flag), (unsigned long)m->m_time, m->m_xlines);
             putc('\n', obuf);
          }
-         Fclose(obuf);
+         mx_fs_close(obuf);
          goto jleave;
       }
       fflush(obuf);
       rewind(obuf);
       ftruncate(fileno(obuf), 0);
    }
-   if ((ibuf = setinput(mp, m, NEED_UNSPEC)) == NULL) {
-      Fclose(obuf);
+
+   if((ibuf = setinput(mp, m, NEED_UNSPEC)) == NIL){
+      mx_fs_close(obuf);
       goto jleave;
    }
+
    if (c == 'N')
       goto jdone;
    fseek(obuf, INITSKIP, SEEK_SET);
@@ -360,8 +367,9 @@ jdone:
    }
    if (c == 'B' && USEBITS(m->m_flag) == MREAD)
       m->m_flag |= MFULLYCACHED;
+
 jout:
-   if (Fclose(obuf) != 0) {
+   if(!mx_fs_close(obuf)){
       m->m_flag &= ~MFULLYCACHED;
       unlink(name);
    }
@@ -389,8 +397,9 @@ initcache(struct mailbox *mp)
       goto jleave;
    if (cwget(&cw) == STOP)
       goto jleave;
-   if ((uvfp = Fopen(uvname, "r+")) == NULL ||
-         (n_file_lock(fileno(uvfp), FLT_READ, 0,0, 0), 0) ||
+
+   if((uvfp = mx_fs_open(uvname, "r+")) == NIL ||
+         (mx_file_lock(fileno(uvfp), mx_FILE_LOCK_TYPE_READ, 0,0, 0), 0) ||
          fscanf(uvfp, "%" PRIu64 , &uv) != 1 || uv != mp->mb_uidvalidity) {
       if ((uvfp = clean(mp, &cw)) == NULL)
          goto jout;
@@ -398,12 +407,21 @@ initcache(struct mailbox *mp)
       fflush(uvfp);
       rewind(uvfp);
    }
-   n_file_lock(fileno(uvfp), FLT_WRITE, 0,0, 0);
+
+   mx_file_lock(fileno(uvfp), mx_FILE_LOCK_TYPE_WRITE, 0,0, 0);
    fprintf(uvfp, "%" PRIu64 "\n", mp->mb_uidvalidity);
-   if (ferror(uvfp) || Fclose(uvfp) != 0) {
-      unlink(uvname);
-      mp->mb_uidvalidity = 0;
+
+   /* C99 */{
+      int x;
+
+      x = ferror(uvfp);
+
+      if(!mx_fs_close(uvfp) || x){
+         unlink(uvname);
+         mp->mb_uidvalidity = 0;
+      }
    }
+
 jout:
    cwrelse(&cw);
 jleave:
@@ -441,26 +459,26 @@ clean(struct mailbox *mp, struct cw *cw)
    if ((cachedir = ok_vlook(imap_cache)) == NULL ||
          (cachedir = fexpand(cachedir, FEXP_LOCAL | FEXP_NOPROTO)) == NULL)
       goto jleave;
-   eaccount = urlxenc(mp->mb_imap_account, TRU1);
+   eaccount = mx_url_xenc(mp->mb_imap_account, TRU1);
    if (su_cs_cmp_case(emailbox = mp->mb_imap_mailbox, "INBOX")) {
       boole err;
 
       emailbox = imap_path_encode(emailbox, &err);
       if(err)
          goto jleave;
-      emailbox = urlxenc(emailbox, TRU1);
+      emailbox = mx_url_xenc(emailbox, TRU1);
    }
    buf = n_autorec_alloc(bufsz = su_cs_len(cachedir) + su_cs_len(eaccount) +
          su_cs_len(emailbox) + 40);
    if (!n_path_mkdir(cachedir))
       goto jleave;
    snprintf(buf, bufsz, "%s/README", cachedir);
-   if ((fp = Fopen(buf, "wx")) != NULL) {
+   if((fp = mx_fs_open(buf, "wx")) != NIL){
       fputs(README1, fp);
       fputs(README2, fp);
       fputs(README3, fp);
       fputs(README4, fp);
-      Fclose(fp);
+      mx_fs_close(fp);
    }
    fp = NULL;
    snprintf(buf, bufsz, "%s/%s/%s", cachedir, eaccount, emailbox);
@@ -477,7 +495,7 @@ clean(struct mailbox *mp, struct cw *cw)
       unlink(dp->d_name);
    }
    closedir(dirp);
-   fp = Fopen("UIDVALIDITY", "w");
+   fp = mx_fs_open("UIDVALIDITY", "w");
 jout:
    if (cwret(cw) == STOP) {
       n_err(_("Fatal: Cannot change back to current directory.\n"));
@@ -668,7 +686,7 @@ cache_list(struct mailbox *mp, const char *base, int strip, FILE *fp)
    if ((cachedir = ok_vlook(imap_cache)) == NULL ||
          (cachedir = fexpand(cachedir, FEXP_LOCAL | FEXP_NOPROTO)) == NULL)
       goto jleave;
-   eaccount = urlxenc(mp->mb_imap_account, TRU1);
+   eaccount = mx_url_xenc(mp->mb_imap_account, TRU1);
    name = n_autorec_alloc(namesz = su_cs_len(cachedir) +
          su_cs_len(eaccount) + 2);
    snprintf(name, namesz, "%s/%s", cachedir, eaccount);
@@ -677,7 +695,7 @@ cache_list(struct mailbox *mp, const char *base, int strip, FILE *fp)
    while ((dp = readdir(dirp)) != NULL) {
       if (dp->d_name[0] == '.')
          continue;
-      cp = cp2 = imap_path_decode(urlxdec(dp->d_name), NULL);
+      cp = cp2 = imap_path_decode(mx_url_xdec(dp->d_name), NULL);
       for (bp = base; *bp && *bp == *cp2; bp++)
          cp2++;
       if (*bp)
@@ -772,12 +790,12 @@ cached_uidvalidity(struct mailbox *mp)
       uv = 0;
       goto jleave;
    }
-   if ((uvfp = Fopen(uvname, "r")) == NULL ||
-         (n_file_lock(fileno(uvfp), FLT_READ, 0,0, 0), 0) ||
+   if((uvfp = mx_fs_open(uvname, "r")) == NIL ||
+         (mx_file_lock(fileno(uvfp), mx_FILE_LOCK_TYPE_READ, 0,0, 0), 0) ||
          fscanf(uvfp, "%" PRIu64, &uv) != 1)
       uv = 0;
-   if (uvfp != NULL)
-      Fclose(uvfp);
+   if(uvfp != NIL)
+      mx_fs_close(uvfp);
 jleave:
    NYD_OU;
    return uv;
@@ -792,8 +810,8 @@ cache_queue1(struct mailbox *mp, char const *mode, char **xname)
 
    if ((name = encname(mp, "QUEUE", 0, NULL)) == NULL)
       goto jleave;
-   if ((fp = Fopen(name, mode)) != NULL)
-      n_file_lock(fileno(fp), FLT_WRITE, 0,0, 0);
+   if((fp = mx_fs_open(name, mode)) != NIL)
+      mx_file_lock(fileno(fp), mx_FILE_LOCK_TYPE_WRITE, 0,0, 0);
    if (xname)
       *xname = name;
 jleave:
@@ -827,7 +845,7 @@ cache_dequeue(struct mailbox *mp)
    if ((cachedir = ok_vlook(imap_cache)) == NULL ||
          (cachedir = fexpand(cachedir, FEXP_LOCAL | FEXP_NOPROTO)) == NULL)
       goto jleave;
-   eaccount = urlxenc(mp->mb_imap_account, TRU1);
+   eaccount = mx_url_xenc(mp->mb_imap_account, TRU1);
    buf = n_autorec_alloc(bufsz = su_cs_len(cachedir) +
          su_cs_len(eaccount) + 2);
    snprintf(buf, bufsz, "%s/%s", cachedir, eaccount);
@@ -840,7 +858,7 @@ cache_dequeue(struct mailbox *mp)
       /* FIXME MUST BLOCK SIGNALS IN ORDER TO ENSURE PROPER RESTORE!
        * (but wuuuuh, what a shit!) */
       mp->mb_imap_mailbox = su_cs_dup(
-            imap_path_decode(urlxdec(dp->d_name), NULL), 0);
+            imap_path_decode(mx_url_xdec(dp->d_name), NULL), 0);
       dequeue1(mp);
       {  char *x = mp->mb_imap_mailbox;
          mp->mb_imap_mailbox = oldbox;
@@ -872,8 +890,8 @@ dequeue1(struct mailbox *mp)
          goto jsave;
       }
       if ((uvname = encname(mp, "UIDVALIDITY", 0, NULL)) == NULL ||
-            (uvfp = Fopen(uvname, "r")) == NULL ||
-            (n_file_lock(fileno(uvfp), FLT_READ, 0,0, 0), 0) ||
+            (uvfp = mx_fs_open(uvname, "r")) == NIL ||
+            (mx_file_lock(fileno(uvfp), mx_FILE_LOCK_TYPE_READ, 0,0, 0), 0) ||
             fscanf(uvfp, "%" PRIu64, &uv) != 1 || uv != mp->mb_uidvalidity) {
          n_err(_("Unique identifiers for \"%s\" are out of date. "
             "Cannot commit IMAP commands.\n"), mp->mb_imap_mailbox);
@@ -881,18 +899,19 @@ jsave:
          n_err(_("Saving IMAP commands to *DEAD*\n"));
          savedeadletter(fp, 0);
          ftruncate(fileno(fp), 0);
-         Fclose(fp);
-         if (uvfp)
-            Fclose(uvfp);
+         mx_fs_close(fp);
+         if(uvfp != NIL)
+            mx_fs_close(uvfp);
          rv = STOP;
          goto jleave;
       }
-      Fclose(uvfp);
+      mx_fs_close(uvfp);
       printf("Committing IMAP commands for \"%s\"\n", mp->mb_imap_mailbox);
       imap_dequeue(mp, fp);
    }
-   if (fp) {
-      Fclose(fp);
+
+   if(fp != NIL){
+      mx_fs_close(fp);
       unlink(qname);
    }
 jleave:
